''' Module to manage street smart settings '''
import os
import logging
from base64 import urlsafe_b64encode, urlsafe_b64decode
import json
from PyQt5.QtWidgets import (  # pylint: disable=no-name-in-module
    QComboBox, QDialog, QDialogButtonBox, QFormLayout,
    QGroupBox, QLabel, QLineEdit, QMessageBox,QLineEdit,QHBoxLayout,
    QVBoxLayout, QTabWidget, QPushButton, QCheckBox, QTextEdit,QDoubleSpinBox,QAbstractSpinBox,
)
from qgis.PyQt.QtCore import (  # pylint: disable=import-error
    QCoreApplication,QSettings
)
from ..logger import Logger
from ..logger import log
from ..street_smart import AbstractCommand
from ..settings import Settings
#from qgis.core import QgsMessageLog


# importing libraries
from PyQt5.QtWidgets import *
from PyQt5 import QtCore, QtGui
from PyQt5.QtGui import *
from PyQt5.QtCore import *


logger = Logger(__name__).get()


class ManageSettingsCommand(AbstractCommand):
    '''
    Class for Manage Settings Command
    '''
    def __init__(self, iface, streetsmart):
        '''
        Constructor
        '''
        super().__init__(iface, streetsmart)

    @staticmethod
    def icon_path():
        '''
        Path for the icon
        '''
        return ':/plugins/street_smart/resources/mActionOptions.svg'

    def text(self):
        '''
        Text for the command to show
        '''
        return super().tr('Settings Street Smart Plugin')

    @staticmethod
    def callback():
        '''
        Code to execute when command is clicked
        '''
        dialog = SettingsDialog(Settings.getInstance())
        dialog.exec_()

    def parent(self):
        '''
        Parent
        '''
        return self.iface.mainWindow()


class SettingsDialogModel():
    ''' Model '''
    def __init__(self, settings: Settings):
        self.settings = settings

    def old_settings(self):
        ''' Original Settings '''

    def new_settings(self):
        ''' Changed Settings '''


def tr(message: str) -> str:
    return QCoreApplication.translate('StreetSmart', message)


class SettingsDialog(QDialog):  # pylint: disable=too-few-public-methods
    ''' Manage settings GUI '''
    def __init__(self, settings):
        super(SettingsDialog, self).__init__()
        self.setWindowTitle(tr("Manage Settings"))

        self._settings = settings
        self._old_settings = self.__get_settings(settings)
        self._new_settings = {}
        self.__configuration_url_widget = None
        self.__streetsmart_location_widget = None
        self.__streetsmart_recordings_wfs_location_widget = None
        self.__proxy_server_domain_widget = None
        self.__proxy_server_password_widget = None
        self.__proxy_server_username_widget = None
        self.__proxy_server_port_widget = None
        self.__proxy_server_address_widget = None

        self.setMinimumWidth(600)
        mainLayout = QVBoxLayout()
        mainLayout.addWidget(self.__create_tab_widget())
        mainLayout.addWidget(self.__create_button_box())
        self.setLayout(mainLayout)

    def __create_button_box(self):
        """ Creates the buttons for the user interface """
        buttonBox = QDialogButtonBox(QDialogButtonBox.Ok
                                     | QDialogButtonBox.Cancel)
        buttonBox.accepted.connect(self._accept_settings)
        buttonBox.rejected.connect(self._reject_settings)

        return buttonBox

    def __create_tab_widget(self):
        ''' Create Tab '''
        tabs = QTabWidget()
        tabs.addTab(self._create_login_tab(), tr("Login"))
        #tabs.addTab(self._create_language_tab(), tr("Language"))
        tabs.addTab(self.__create_configuration_tab(), tr("Configuration"))
        tabs.addTab(self.__create_advance_configuration_tab(), tr("Advanced Configuration"))
        tabs.addTab(self._create_about_tab(), tr("About"))
        tabs.addTab(self._create_agreement_tab(), tr("Agreement"))

        return tabs

    @staticmethod
    def __get_settings(settings):
        ''' Save Settings '''
        result = {}
        for key in Settings.ALL:
            if key in settings.settings:
                print("Keep track of attribute {} with value {}".format(key, settings.settings[key].value))
                result[key] = settings.settings[key].value
            else:
                # Handle keys that aren't yet in the settings dictionary
                print(f"Setting {key} not found, using default value")
                result[key] = None
        return result

    def __save_settings(self):
        ''' Save the changed settings '''
        for key in self._new_settings:
            logger.debug(f"Update setting, test {key}, new: {self._new_settings[key]},was: {self._old_settings[key]}")
            if self._new_settings[key] != self._old_settings[key]:
                logger.debug("Update setting %s %s -> %s", key, self._old_settings[key], self._new_settings[key])
                self._settings.update_setting(key, self._new_settings[key])
        self._settings.save()

    def _create_login_tab(self):
        ''' Login '''
        username = self._old_settings[Settings.USERNAME]
        password = self._old_settings[Settings.PASSWORD]
        groupbox = QGroupBox(tr("Login"))
        layout = QFormLayout()
        e = QLineEdit(username)
        # e.setText(username)
        e.textChanged.connect(self.__username_changed)
        layout.addRow(QLabel(tr("Username:")), e)
        pwd_edit = QLineEdit(password)
        pwd_edit.setEchoMode(QLineEdit.Password)
        # pwd_edit.setText(password)
        pwd_edit.textChanged.connect(self.__password_changed)
        layout.addRow(QLabel(tr("Password:")), pwd_edit)
        # status_label = QLabel(tr("Status not checked"))
        # layout.addRow(QLabel(tr("Status:")), status_label)
        # check_button = QPushButton(tr("Check"))
        # check_button.clicked.connect(self.check_clicked)
        # layout.addRow(None, check_button)
        groupbox.setLayout(layout)
        return groupbox

    def __username_changed(self, text):
        ''' h '''
        self._new_settings[Settings.USERNAME] = text
        print(text)

    def __password_changed(self, text):
        ''' h '''
        self._new_settings[Settings.PASSWORD] = text
        print(text)

    @staticmethod
    def check_clicked():
        ''' button check slot '''
        raise NotImplementedError("Check is not yet implemented")

    def _create_language_tab(self):
        ''' Language '''
        groupbox = QGroupBox(tr("Language"))
        layout = QFormLayout()
        e = QComboBox()
        e.addItems(["en-us", "nl-BE", "nl-NL"])
        e.setCurrentText("en-us")
        e.currentIndexChanged.connect(self.__language_changed)
        layout.addRow(QLabel(tr("Language:")), e)
        groupbox.setLayout(layout)
        return groupbox

    def __language_changed(self, text):
        ''' Language changed '''
        self._new_settings[Settings.LANGUAGE] = text
        print("Language changed: ", text)

    @staticmethod
    @log(logging.DEBUG, print_args=True)
    def __set_widget_enable_state(widget, enable):
        """ Sets the enabled state for a widget """
        print("Set widget enabled to ", enable)
        widget.setEnabled(enable)
        print("Enabled state is ", widget.isEnabled())

    @staticmethod
    @log(logging.DEBUG, print_args=True)
    def __set_widget_disable_state(widget, disable):
        """ Sets the enabled state for a widget """
        print("Set widget enabled to ", disable)
        widget.setDisabled(True)
        print("Enabled state is ", widget.setDisabled(True))



    def __create_configuration_box(self):
        """ Returns streetsmart configuration groupbox """

        def __configuration_url_changed(text):
            ''' h '''
            self._new_settings[Settings.CONFIGURATION_URL] = text
            logger.debug(f"Configuration URL in box changed: f{text}")

        @log(logging.DEBUG, print_args=True)
        def __use_default_url_changed(state):
            ''' Default url changed '''
            logger.debug(f"Use Default Configuration Checkbox State changed {state}")
            self._new_settings[Settings.USE_DEFAULT_CONFIGURATION_URL] = state
            if int(state) == 2:
                self._new_settings[Settings.CONFIGURATION_URL] = None
            self.__set_widget_enable_state(self.__configuration_url_widget,
                                           not state)

        config_groupbox = QGroupBox(tr("Config URL"))
        layout = QFormLayout()
        chk = QCheckBox()
        chk.stateChanged.connect(__use_default_url_changed)
        le = QLineEdit()
        le.textChanged.connect(__configuration_url_changed)
        self.__configuration_url_widget = le
        layout.addRow(chk, QLabel(tr("Use default")))
        layout.addRow(QLabel(tr("Location")), le)
        config_groupbox.setLayout(layout)

        # Set Initial Values
        use_default_configuration_url = self._old_settings[Settings.USE_DEFAULT_CONFIGURATION_URL]  # pylint: disable=line-too-long
        configuration_url = self._old_settings[Settings.CONFIGURATION_URL]
        if use_default_configuration_url:
            chk.setCheckState(2)
            le.setText("")
        else:
            chk.setCheckState(0)
            le.setText(configuration_url)

        return config_groupbox

    def __create_api_location_box(self):
        """ Returns streetsmart location configuration groupbox """

        def __use_default_location_changed(state):
            ''' Default streetsmart location checkbox changed '''
            self._new_settings[Settings.USE_DEFAULT_STREETSMART_LOCATION] = state
            if state == 2:
                self._new_settings[Settings.STREETSMART_LOCATION] = None
            self.__set_widget_enable_state(self.__streetsmart_location_widget,
                                           not state)

        def __location_url_changed(text):
            ''' h '''
            self._new_settings[Settings.STREETSMART_LOCATION] = text
            print(text)

        streetsmart_groupbox = QGroupBox(tr("StreetSmart URL"))
        layout = QFormLayout()
        chk = QCheckBox()
        chk.stateChanged.connect(__use_default_location_changed)
        le = QLineEdit()
        le.textChanged.connect(__location_url_changed)
        self.__streetsmart_location_widget = le
        layout.addRow(chk, QLabel(tr("Use default")))
        layout.addRow(QLabel(tr("Location")), le)
        streetsmart_groupbox.setLayout(layout)

        # Set Initial Values
        use_default_streetsmart_loc = self._old_settings[Settings.USE_DEFAULT_STREETSMART_LOCATION]  # pylint: disable=line-too-long
        streetsmart_location = self._old_settings[Settings.STREETSMART_LOCATION]
        if use_default_streetsmart_loc:
            chk.setCheckState(2)
            le.setText("")
        else:
            chk.setCheckState(0)
            le.setText(streetsmart_location)

        return streetsmart_groupbox

    def __create_recording_location_box(self):
        """ Returns recordings wgfs location configuration groupbox """

        def __use_default_recordings_wfs_location_changed(state):
            ''' Default recordings wfs location checkbox changed '''
            self._new_settings[Settings.USE_DEFAULT_RECORDING_WFS_LOCATION] = state
            if state == 2:
                self._new_settings[Settings.RECORDING_WFS_LOCATION] = None
            self.__set_widget_enable_state(self.__streetsmart_recordings_wfs_location_widget,
                                           not state)

        def __recordings_wfs_location_url_changed(text):
            ''' h '''
            self._new_settings[Settings.RECORDING_WFS_LOCATION] = text
            print(text)

        recording_wfs_groupbox = QGroupBox(tr("Recordings WFS URL"))
        layout = QFormLayout()
        chk = QCheckBox()
        chk.stateChanged.connect(__use_default_recordings_wfs_location_changed)
        le = QLineEdit()
        le.textChanged.connect(__recordings_wfs_location_url_changed)
        self.__streetsmart_recordings_wfs_location_widget = le
        layout.addRow(chk, QLabel(tr("Use default")))
        layout.addRow(QLabel(tr("Location")), le)
        recording_wfs_groupbox.setLayout(layout)

        # Set Initial Values
        use_default_recordings_wfs_loc = self._old_settings[Settings.USE_DEFAULT_RECORDING_WFS_LOCATION]  # pylint: disable=line-too-long
        recordings_wfs_location = self._old_settings[Settings.RECORDING_WFS_LOCATION]
        if use_default_recordings_wfs_loc:
            chk.setCheckState(2)
            le.setText("")
        else:
            chk.setCheckState(0)
            le.setText(recordings_wfs_location)

        return recording_wfs_groupbox

    def __create_proxy_box(self):
        """ Returns proxy server configuration groupbox """
        proxy_groupbox = QGroupBox(tr("Proxy Server"))
        layout = QFormLayout()
        chk = QCheckBox()
        chk.stateChanged.connect(self.__use_proxy_server_changed)
        layout.addRow(chk, QLabel(tr("Use a proxy server")))
        le_address = QLineEdit()
        layout.addRow(QLabel(tr("Address")), le_address)
        self.__proxy_server_address_widget = le_address
        le_port = QLineEdit()
        layout.addRow(QLabel(tr("Port")), le_port)
        self.__proxy_server_port_widget = le_port

        # Set Initial Values
        use_proxy_server = self._old_settings[Settings.USE_PROXY_SERVER]  # pylint: disable=line-too-long
        proxy_address = self._old_settings[Settings.PROXY_ADDRESS]
        proxy_port = self._old_settings[Settings.PROXY_PORT]
        if use_proxy_server:
            chk.setCheckState(0)
            le_address.setText("")
            le_port.setText("")
        else:
            chk.setCheckState(2)
            le_address.setText(proxy_address)
            le_port.setText(proxy_port)

        chk = QCheckBox()
        chk.stateChanged.connect(self.__bypass_proxy_local_changed)
        layout.addRow(chk, QLabel(tr("Bypass proxy for local addresses")))
        # Set Initial Values
        bypass_local_addresses = self._old_settings[Settings.BYPASS_PROXY_FOR_LOCAL_ADDRESSES]  # pylint: disable=line-too-long
        if bypass_local_addresses:
            chk.setCheckState(2)
        else:
            chk.setCheckState(0)

        chk = QCheckBox()
        chk.stateChanged.connect(self.__use_default_proxy_credentials_changed)
        layout.addRow(chk, QLabel(tr("Use default credentials")))
        le_username = QLineEdit()
        layout.addRow(QLabel(tr("Username")), le_username)
        self.__proxy_server_username_widget = le_username
        le_password = QLineEdit()
        layout.addRow(QLabel(tr("Password")), le_password)
        self.__proxy_server_password_widget = le_password
        le_domain = QLineEdit()
        layout.addRow(QLabel(tr("Domain")), le_domain)
        self.__proxy_server_domain_widget = le_domain

        # Set Initial Values
        use_default_proxy_credentials = self._old_settings[Settings.USE_DEFAULT_PROXY_CREDENTIALS]  # pylint: disable=line-too-long
        proxy_username = self._old_settings[Settings.PROXY_USERNAME]
        proxy_password = self._old_settings[Settings.PROXY_PASSWORD]
        proxy_domain = self._old_settings[Settings.PROXY_DOMAIN]
        if use_default_proxy_credentials:
            chk.setCheckState(2)
            le_username.setText("")
            le_password.setText("")
            le_domain.setText("")
        else:
            chk.setCheckState(0)
            le_username.setText(proxy_username)
            le_password.setText(proxy_password)
            le_domain.setText(proxy_domain)
        proxy_groupbox.setLayout(layout)
        return proxy_groupbox

    def __create_configuration_tab(self):
        ''' Configuration '''
        main_groupbox = QGroupBox(tr("Configuration"))
        main_layout = QVBoxLayout()

        main_layout.addWidget(self.__create_configuration_box())
        main_layout.addWidget(self.__create_api_location_box())
        main_layout.addWidget(self.__create_recording_location_box())
        # main_layout.addWidget(self.__create_proxy_box())

        main_groupbox.setLayout(main_layout)
        return main_groupbox

    def __use_proxy_server_changed(self, state):
        ''' Default url changed '''
        self._new_settings[Settings.USE_PROXY_SERVER] = state
        self.__set_widget_enable_state(self.__proxy_server_port_widget,
                                       not state)
        self.__set_widget_enable_state(self.__proxy_server_address_widget,
                                       not state)

    def __use_default_proxy_credentials_changed(self, state):
        ''' Default url changed '''
        self._new_settings[Settings.USE_PROXY_SERVER] = state
        self.__set_widget_enable_state(self.__proxy_server_username_widget,
                                       not state)
        self.__set_widget_enable_state(self.__proxy_server_password_widget,
                                       not state)
        self.__set_widget_enable_state(self.__proxy_server_domain_widget,
                                       not state)

    def __bypass_proxy_local_changed(self, state):
        ''' Default url changed '''
        self._new_settings[Settings.BYPASS_PROXY_FOR_LOCAL_ADDRESSES] = state


    def __create_filter_on_XYZ_location_box(self):
        """Returns streetsmart location configuration groupbox"""

        def __use_default_precision_filter_changed(state):
            """Default streetsmart precision filter checkbox changed"""
            is_checked = state == 2  # Qt.Checked = 2, Qt.Unchecked = 0
            
            # Save state to QSettings directly
            settings = QSettings()
            settings.setValue("StreetSmart/use_default_precision_filter", is_checked)
            
            # Update precision values based on checkbox state
            if is_checked:
                # When using defaults, set precision values to None
                self._new_settings[Settings.LATITUDE_PRECISION] = None
                self._new_settings[Settings.LONGITUDE_PRECISION] = None
                self._new_settings[Settings.HEIGHT_PRECISION] = None
                
                # Update settings directly
                self._settings.update_setting(Settings.LATITUDE_PRECISION, None)
                self._settings.update_setting(Settings.LONGITUDE_PRECISION, None)
                self._settings.update_setting(Settings.HEIGHT_PRECISION, None)
                self._settings.save()
            else:
                # When not using defaults, save the current values
                self._new_settings[Settings.LATITUDE_PRECISION] = self.__streetsmart_latitude_precision_widget.value()
                self._new_settings[Settings.LONGITUDE_PRECISION] = self.__streetsmart_longitude_precision_widget.value()
                self._new_settings[Settings.HEIGHT_PRECISION] = self.__streetsmart_height_precision_widget.value()
            
            # Update UI controls
            self.__set_widget_enable_state(self.__streetsmart_latitude_precision_widget, not is_checked)
            self.__set_widget_enable_state(self.__streetsmart_longitude_precision_widget, not is_checked)
            self.__set_widget_enable_state(self.__streetsmart_height_precision_widget, not is_checked)



        # Add handlers for spinbox value changes - each saves its value directly to settings
        def __latitude_value_changed(value):
            """Handle latitude value change - update ONLY latitude"""
            if not self.chk.isChecked():  # Only update if not using defaults
                # Only update the latitude value
                __set_precision_value("latitude_precision", value)
                # Update the internal settings tracking
                self._new_settings[Settings.LATITUDE_PRECISION] = value
                print(f"Latitude precision changed and saved to: {value}")

        def __longitude_value_changed(value):
            """Handle longitude value change - update ONLY longitude"""
            if not self.chk.isChecked():  # Only update if not using defaults
                # Only update the longitude value
                __set_precision_value("longitude_precision", value) 
                # Update the internal settings tracking
                self._new_settings[Settings.LONGITUDE_PRECISION] = value
                print(f"Longitude precision changed and saved to: {value}")

        def __height_value_changed(value):
            """Handle height value change - update ONLY height"""
            if not self.chk.isChecked():  # Only update if not using defaults
                # Only update the height value
                __set_precision_value("height_precision", value)
                # Update the internal settings tracking
                self._new_settings[Settings.HEIGHT_PRECISION] = value
                print(f"Height precision changed and saved to: {value}")



        def __get_precision_value(key, default=0.001):
            """Get precision value directly from QSettings"""
            from PyQt5.QtCore import QSettings
            settings = QSettings()
            # Use consistent casing: "StreetSmart" not "streetsmart" 
            return settings.value(f"StreetSmart/{key}", default, type=float)

        def __set_precision_value(key, value):
            """Set precision value directly to QSettings"""
            from PyQt5.QtCore import QSettings
            settings = QSettings()
            # Use consistent casing: "StreetSmart" not "streetsmart"
            settings.setValue(f"StreetSmart/{key}", value)
            settings.sync()
            print(f"Set precision value {key} to {value}")


    
        def __toggle_attribute_state_changed(state, chk):
            """Handle attribute visibility toggle"""
            is_checked = state == 2  # Qt.Checked = 2, Qt.Unchecked = 0
            self._new_settings[Settings.ATTRIBUTE_STATE] = is_checked
            self.save_attribute_state("Streetsmart", is_checked)  # Persist the state
            chk.setText(tr("Show Attribute on Measure") if is_checked else tr("Hide Attribute on Measure"))
            logger.debug(f"Attribute visibility toggled: {is_checked}")

        streetsmart_groupbox = QGroupBox(tr("Filter on location"))
        layout = QFormLayout()
        
        # Create the "Use default" checkbox
        self.chk = QCheckBox()
        self.chk.stateChanged.connect(__use_default_precision_filter_changed)

        # Initialize spin boxes
        latitude_spinbox = QDoubleSpinBox()
        longitude_spinbox = QDoubleSpinBox()
        height_spinbox = QDoubleSpinBox()
        
        # Configure spin boxes
        latitude_spinbox.setDecimals(3)
        longitude_spinbox.setDecimals(3)
        height_spinbox.setDecimals(3)
        latitude_spinbox.setSingleStep(0.1)
        longitude_spinbox.setSingleStep(0.1)
        height_spinbox.setSingleStep(0.1)
        
        # Connect value changed signals for spin boxes - use editingFinished to avoid too many updates
        latitude_spinbox.valueChanged.connect(__latitude_value_changed)
        longitude_spinbox.valueChanged.connect(__longitude_value_changed)
        height_spinbox.valueChanged.connect(__height_value_changed)

        # Save references
        self.__streetsmart_latitude_precision_widget = latitude_spinbox
        self.__streetsmart_longitude_precision_widget = longitude_spinbox
        self.__streetsmart_height_precision_widget = height_spinbox

        # Create attribute toggle checkbox
        toggle_attribute_chk = QCheckBox()
        toggle_attribute_chk.stateChanged.connect(lambda state: __toggle_attribute_state_changed(state, toggle_attribute_chk))

        # ======= DETERMINE INITIAL STATE FOR CHECKBOXES AND SPIN BOXES =======
        
        settings = QSettings()
        use_default_filter = settings.value("StreetSmart/use_default_precision_filter", False, type=bool)

        # When setting initial spinbox values, use the get_precision_value method:
        latitude_spinbox.setValue(__get_precision_value("latitude_precision"))
        longitude_spinbox.setValue(__get_precision_value("longitude_precision"))
        height_spinbox.setValue(__get_precision_value("height_precision"))
        


        
        # Set checkbox state based on saved value
        if use_default_filter:
            self.chk.setCheckState(Qt.Checked)  # 2
            # Disable spin boxes when using defaults
            latitude_spinbox.setEnabled(False)
            longitude_spinbox.setEnabled(False)
            height_spinbox.setEnabled(False)
        else:
            self.chk.setCheckState(Qt.Unchecked)  # 0
        
        # Get precision values from settings
        #latitude_value = self._old_settings.get(Settings.LATITUDE_PRECISION, 0.001)
        #longitude_value = self._old_settings.get(Settings.LONGITUDE_PRECISION, 0.001)
        #height_value = self._old_settings.get(Settings.HEIGHT_PRECISION, 0.001)
        
        # Debug output
        #print(f"Loading saved values - Latitude: {latitude_value}, Longitude: {longitude_value}, Height: {height_value}")
        
        # Set spin box values (safely convert to float)
        #latitude_spinbox.setValue(self.convert_to_float(latitude_value, 0.001))
        #longitude_spinbox.setValue(self.convert_to_float(longitude_value, 0.001))
        #height_spinbox.setValue(self.convert_to_float(height_value, 0.001))

        # Set initial state for attribute toggle
        attribute_state = self.get_attribute_state("Streetsmart")
        toggle_attribute_chk.setCheckState(Qt.Checked if attribute_state else Qt.Unchecked)
        toggle_attribute_chk.setText(tr("Show Attribute on Measure") if attribute_state else tr("Hide Attribute on Measure"))

        # Layout setup
        layout.addRow(self.chk, QLabel(tr("Use default")))
        layout.addRow(QLabel(tr("Latitude X")), latitude_spinbox)
        layout.addRow(QLabel(tr("Longitude Y")), longitude_spinbox)
        layout.addRow(QLabel(tr("Height Z")), height_spinbox)
        layout.addRow(QFrame(frameShape=QFrame.HLine, frameShadow=QFrame.Sunken))  # Separator
        layout.addRow(toggle_attribute_chk)

        streetsmart_groupbox.setLayout(layout)
        return streetsmart_groupbox



    def convert_to_float(self,value, default=0.0):
        """
        Converts a value to float, with error handling.
        
        Args:
            value: The value to convert, can be any type
            default: Default value if conversion fails, defaults to 0.0
            
        Returns:
            float: The converted float value or the default if conversion fails
        """
        if isinstance(value, str):
            try:
                return float(value)
            except ValueError:
                return default
        elif isinstance(value, (int, float)):
            return float(value)
        else:
            return default
    

    @log(logging.DEBUG, print_args=True)
    def __use_default_attribute_state_changed(self, state, chk):
        ''' Default attribute state changed '''
        logger.debug(f"Use Default attribute state Checkbox State changed: {state}")
        
        # Save the current state to both the settings dialog and persistent storage
        is_checked = chk.isChecked()
        self._new_settings[Settings.ATTRIBUTE_STATE] = is_checked
        self.save_attribute_state("Streetsmart", is_checked)
        print("Streetsmart state", is_checked)
        
        # Update the checkbox text to match its current state
        if is_checked:
            chk.setText(tr("Show Attribute on Measure"))
        else:
            chk.setText(tr("Hide Attribute on Measure"))
        
        print("Attributes visibility toggled:", is_checked)

    def __latitude_changed(self, value):
        """Handle latitude precision change"""
        self._new_settings[Settings.LATITUDE_PRECISION] = value
        logger.debug(f"Latitude precision changed: {value}")

    def get_attribute_state(self, service_name):
        """Get the saved attribute visibility state from QSettings"""
        settings = QSettings()
        return settings.value(f"{service_name}/attribute_state", False, type=bool)

    def save_attribute_state(self, service_name, is_checked):
        """Save the attribute visibility state to QSettings"""
        settings = QSettings()
        settings.setValue(f"{service_name}/attribute_state", is_checked)
        settings.sync()
    
    def toggle_attributes_visibility(self,chk):

                    # Set Initial Values
        use_default_attribute_state= self._old_settings[Settings.ATTRIBUTE_STATE]  # pylint: disable=line-too-long
        
        if use_default_attribute_state:
            chk.setCheckState(2)
            chk.setText(tr("Show Attribute on Measure"))
        else:
            chk.setText(tr("Hide Attribute on Measure"))
        print("Attributes visibility toggled:", chk.isChecked())



    def encode(self,data):
        return urlsafe_b64encode(bytes(data, 'utf-8'))

    def decode(self,enc):
        return urlsafe_b64decode(enc).decode()
    
    def read_json(self,path):
        with open(path, "r") as handler:
            info = json.load(handler)
        
        return info

        
        #return line
    def crete_Qline_edit(self):
            self.inputLabel = QLabel("Passphrase")
            self.editLine = QLineEdit()
            self.OKButton = QPushButton("OK")

            #self.printButton.clicked.connect(self.printText)
            #self.clearButton.clicked.connect(self.clearText)

            inputLayout = QHBoxLayout()
            inputLayout.addWidget(self.inputLabel)
            inputLayout.addWidget(self.editLine)

            buttonLayout = QHBoxLayout()
            buttonLayout.addWidget(self.OKButton)

            mainLayout = QVBoxLayout()
            mainLayout.addLayout(inputLayout)
            mainLayout.addLayout(buttonLayout)

            #self.setLayout(mainLayout)
            #self.setWindowTitle('FristWindow')
            #self.show()  
            #return mainLayout   




    def __create_advance_configuration_tab(self):
        ''' Advanced Configuration '''
        main_groupboxAd = QGroupBox(tr("Advanced Configuration"))

        main_layoutAd = QVBoxLayout()

        main_layoutAd.addWidget(self.__create_filter_on_XYZ_location_box())
        #main_groupboxAd.setEnabled(True)
        #if Settings.USERNAME == "john.joosten@merkator.com":
            #main_groupboxAd.setEnabled(True)
        
        main_groupboxAd.setLayout(main_layoutAd)
        return main_groupboxAd

    
    def get_values_from_filter_on_location_boxes(self):
        """Get current spinbox values for query building without updating settings"""
        # Always get values directly from spinboxes for accurate query building
        latitude_precision = self.__streetsmart_latitude_precision_widget.value()
        longitude_precision = self.__streetsmart_longitude_precision_widget.value()
        height_precision = self.__streetsmart_height_precision_widget.value()
        
        # Print current values to help with debugging
        print(f"Current values for query - Lat: {latitude_precision}, Long: {longitude_precision}, Height: {height_precision}")
        
        # Build filter query
        if self.chk.isChecked():
            filter_query = "expiredAt is null"
        else:
            filter_query = f"expiredAt is null AND latitudePrecision > {latitude_precision} OR longitudePrecision > {longitude_precision}"
        
        # Send query to apply to Cyclomedia layer
        Settings.retrieve_All_Querry_filter_on_location(filter_query)
        
    def save_attribute_state(self,service_name, is_checked):
        settings = QSettings()
        settings.setValue(f"{service_name}/attribute_state", is_checked)
        settings.sync()  # Ensure the settings are written immediately

    @staticmethod
    def _create_about_tab():
        ''' About '''
        about_html = "<p>Street Smart for QGIS</p>"
        about_html += "<p>Copyright &copy; Merkator 2023</p>"
        about_html += "<p>version v3.2.4.120.2</p>"
        about_html += "<p>git sha 2624fa958dd3bd1c6729e36df552406ad92ad02b</p>"   
        about_html += "<p>http://www.merkator.com</p>"
        e = QTextEdit()
        e.setHtml(about_html)
        e.setReadOnly(True)
        return e

    @staticmethod
    def _create_agreement_tab():
        ''' Agreement '''
        current_path = os.path.dirname(os.path.abspath(__file__))
        parent_directory = os.path.dirname(current_path)
        agreement_file = os.path.join(parent_directory, "agreement.txt")
        with open(agreement_file, "r") as f:
            gpl_license_html = f.readlines()

        gpl_license_html = "".join(gpl_license_html)
        e = QTextEdit()
        e.setHtml(gpl_license_html)
        e.setReadOnly(True)
        return e

    def _is_dirty(self):
        ''' Returns true if there were changes made in the UI '''
        settings = QSettings()
        
        # Define precision keys that are saved immediately and should be excluded
        precision_keys = [
            Settings.LATITUDE_PRECISION,
            Settings.LONGITUDE_PRECISION,
            Settings.HEIGHT_PRECISION,
            Settings.ATTRIBUTE_STATE
        ]
        
        for key in self._new_settings:
            # Skip precision settings as they're managed separately
            if key in precision_keys:
                continue
                
            # First try to get value from QSettings
            stored_value = settings.value(f"StreetSmart/{key}", None)
            
            # If found in QSettings, compare with that
            if stored_value is not None:
                # Handle type conversion for boolean values
                if key in [Settings.USE_DEFAULT_CONFIGURATION_URL,
                        Settings.USE_DEFAULT_STREETSMART_LOCATION,
                        Settings.USE_DEFAULT_RECORDING_WFS_LOCATION,
                        Settings.USE_PROXY_SERVER,
                        Settings.BYPASS_PROXY_FOR_LOCAL_ADDRESSES,
                        Settings.USE_DEFAULT_PROXY_CREDENTIALS]:
                    stored_value = bool(stored_value)
                    
                if self._new_settings[key] != stored_value:
                    logger.debug(f"QSettings change detected in {key}: {stored_value} -> {self._new_settings[key]}")
                    return True
            # Fallback to _old_settings if not found in QSettings
            elif key in self._old_settings:
                if self._old_settings[key] != self._new_settings[key]:
                    logger.debug(f"Old settings change detected in {key}: {self._old_settings[key]} -> {self._new_settings[key]}")
                    return True
                    
        return False

    def _accept_settings(self):
        ''' Accept the changes in the settings without imposing value synchronization '''
        # Build filter query for Cyclomedia layer
        self.get_values_from_filter_on_location_boxes()
        
        if self._is_dirty():
            try:
                btn_reply = QMessageBox.question(self, tr('Manage Settings'),
                                                tr("Are you sure?"),
                                                QMessageBox.Yes | QMessageBox.No,
                                                QMessageBox.No)
                if btn_reply == QMessageBox.Yes:
                    if logger:
                        logger.debug("Save settings")
                    self.__save_settings()
            except Exception as ex:
                if logger:
                    logger.exception("Error occurred saving settings")
                QMessageBox.information(self, tr('Manage Settings'),
                                        tr("Something went wrong.") + str(ex),
                                        QMessageBox.Ok)
                print(ex)
        else:
            QMessageBox.information(self, tr('Manage Settings'),
                                    tr("No changes need to be saved"),
                                    QMessageBox.Ok)
        super().accept()

    def _reject_settings(self):
        ''' Reject changes in the settings '''
        QMessageBox.information(self, tr('Manage Settings'),
                                tr("Settings will not be saved!"),
                                QMessageBox.Ok)
        super().reject()

