# -*- coding: utf-8 -*-
"""
Dependency Manager for StreetSmart QGIS plugin

This module handles package and dependency management for the StreetSmart plugin.
"""
import importlib
import logging
import os
import subprocess
import sys
from pathlib import Path
from dataclasses import dataclass
from typing import Dict, List, Tuple, Optional

from qgis.PyQt.QtWidgets import QMessageBox

# Configure logger
from .logger import Logger
logger = Logger(__name__).get()

# Python executable path
PYTHON_EXECUTABLE = os.path.join(sys.base_exec_prefix, "python.exe")

# Package information
@dataclass
class PackageInfo:
    """Information about a required package"""
    name: str  # Package name for import
    pip_name: str  # Package name for pip (can include version)
    url: Optional[str] = None  # URL for direct install (if needed)

# Required packages
REQUIRED_PACKAGES = {
    "cefpython3": {
        "3.6": PackageInfo("cefpython3", "cefpython3==66.0"),
        "3.9": PackageInfo(
            "cefpython3", 
            "cefpython3==120.2", 
            "https://github.com/Samsonboadi/cefpython-merk-cef120/releases/download/cefpython120.2/cefpython3-120.2-py2.py3-none-win_amd64.whl"
        ),
        "3.12": PackageInfo(
            "cefpython3", 
            "cefpython3==124.3",
            "https://github.com/Samsonboadi/cefpython-merk-cef120/releases/download/cefpython3-124.3/cefpython3-124.3-py2.py3-none-win_amd64.whl"
        )
    },
    "geojson": PackageInfo("geojson", "geojson"),
    "shapely": PackageInfo("shapely", "shapely"),
    "keyring": PackageInfo("keyring", "keyring")
}


def check_python_version() -> Tuple[bool, Optional[str]]:
    """
    Check if the current Python version is supported.
    
    Returns:
        Tuple with (is_supported, error_message)
    """
    python_version = sys.version_info
    major = python_version.major
    minor = python_version.minor
    
    # Define supported versions
    supported_versions = [(3, 6), (3, 9), (3, 12)]
    current_version = (major, minor)
    
    if current_version in supported_versions:
        return True, None
    
    # Create user-friendly message for unsupported version
    supported_versions_str = ", ".join([f"Python {ver[0]}.{ver[1]}" for ver in supported_versions])
    message = (
        f"Unsupported Python Version\n\n"
        f"Your QGIS is using Python {major}.{minor}, which is not supported by this plugin.\n"
        f"This plugin only supports QGIS installations with the following Python versions:\n"
        f"{supported_versions_str}\n\n"
        f"Please use a QGIS version that includes one of these Python versions."
    )
    
    return False, message


def get_python_version_key() -> str:
    """
    Get the Python version key for package lookup.
    
    Returns:
        String representation of Python version (e.g., "3.9")
    """
    python_version = sys.version_info
    return f"{python_version.major}.{python_version.minor}"


def is_package_installed(package_name: str) -> bool:
    """
    Check if a package is installed.
    
    Args:
        package_name: Name of the package to check
        
    Returns:
        True if installed, False otherwise
    """
    return importlib.util.find_spec(package_name) is not None


def install_package(package_info: PackageInfo) -> bool:
    """
    Install a Python package using pip.
    
    Args:
        package_info: PackageInfo object with package details
        
    Returns:
        True if installation succeeded, False otherwise
    """
    logger.info(f"Installing {package_info.name}...")
    
    # Try standard pip install first
    pip_target = package_info.url or package_info.pip_name
    
    try:
        logger.debug(f"Running: {PYTHON_EXECUTABLE} -m pip install --user {pip_target}")
        result = subprocess.run(
            [PYTHON_EXECUTABLE, "-m", "pip", "install", "--user", pip_target],
            check=False,
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
            text=True
        )
        
        if result.returncode == 0:
            logger.info(f"Successfully installed {package_info.name}")
            return True
            
        # If remote install fails, try from local wheels directory
        logger.warning(f"Remote install failed: {result.stderr}")
        logger.info("Trying to install from local wheels directory...")
        
        wheels_dir = os.path.join(os.path.dirname(__file__), "wheels")
        if os.path.exists(wheels_dir):
            result = subprocess.run(
                [
                    PYTHON_EXECUTABLE, "-m", "pip", "install", "--user",
                    "--no-index", f"--find-links={wheels_dir}", package_info.pip_name
                ],
                check=False,
                stderr=subprocess.PIPE,
                stdout=subprocess.PIPE,
                text=True
            )
            
            if result.returncode == 0:
                logger.info(f"Successfully installed {package_info.name} from local wheels")
                return True
                
        logger.error(f"Failed to install {package_info.name}: {result.stderr}")
        return False
        
    except Exception as e:
        logger.error(f"Exception installing {package_info.name}: {str(e)}")
        return False


def install_required_packages(required_packages: Dict) -> bool:
    """
    Install all required packages that are not already installed.
    
    Args:
        required_packages: Dictionary of required packages
        
    Returns:
        True if all requirements are satisfied, False otherwise
    """
    # Check Python version compatibility
    is_supported, error_message = check_python_version()
    if not is_supported:
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Warning)
        msg.setText(error_message)
        msg.setWindowTitle("Python Version Not Supported")
        msg.setStandardButtons(QMessageBox.Ok)
        msg.exec_()
        return False
        
    python_version = get_python_version_key()
    all_installed = True
    
    # Install cefpython3 (version-specific)
    if "cefpython3" in required_packages:
        package_info = required_packages["cefpython3"].get(python_version)
        if package_info:
            if not is_package_installed(package_info.name):
                all_installed = install_package(package_info) and all_installed
        else:
            logger.error(f"No cefpython3 package defined for Python {python_version}")
            all_installed = False
    
    # Install other packages
    for pkg_name, pkg_info in required_packages.items():
        if pkg_name == "cefpython3":
            continue  # Already handled above
            
        if not is_package_installed(pkg_info.name):
            all_installed = install_package(pkg_info) and all_installed
    
    return all_installed


def copy_ssl_modules() -> None:
    """
    Copy SSL DLL files to Python DLLs directory if needed for QGIS.
    This is necessary for SSL to work properly with pip in some QGIS versions.
    """
    from qgis.core import QgsExpressionContextUtils
    
    try:
        qgis_version = QgsExpressionContextUtils.globalScope().variable('qgis_version')
        qgis_version = qgis_version.split('-')[0]
        
        # Find Python directory from QGIS version
        qgis_base_dir = os.path.join(r"C:\Program Files\QGIS " + qgis_version)
        python_dlls_dir = os.path.join(qgis_base_dir, "apps", "Python39", "DLLs")
        bin_dir = os.path.join(qgis_base_dir, "bin")
        
        # Files to copy
        ssl_files = [
            "libcrypto-1_1-x64.dll",
            "libssl-1_1-x64.dll"
        ]
        
        for file in ssl_files:
            source_path = os.path.join(bin_dir, file)
            dest_path = os.path.join(python_dlls_dir, file)
            
            if os.path.exists(source_path) and not os.path.exists(dest_path):
                import shutil
                logger.info(f"Copying {file} to Python DLLs directory")
                shutil.copy2(source_path, dest_path)
    except Exception as e:
        logger.error(f"Error copying SSL modules: {str(e)}")