"""
This module implements the handler for window loaded messages.
"""
import json
import logging

try:
    from .logger import Logger
    from .logger import log
    from .settings import Settings
except ImportError:
    from logger import Logger
    from logger import log
    from settings import Settings

logger = Logger(__name__).get()

class WindowLoadedHandler:
    """
    Handles the window loaded event from the panorama viewer.
    """
    def __init__(self, streetsmart, command_queue=None):
        """
        Initialize the WindowLoadedHandler
        
        Args:
            streetsmart: StreetSmart plugin reference
            command_queue: Optional command queue (not used, but needed for compatibility)
        """
        self.streetsmart = streetsmart
        # Store command_queue reference if needed later
        self.command_queue = command_queue

    def handle(self, _):
        """
        Handles the window loaded event.
        Sets up the JavaScript API for the panorama viewer.
        
        Args:
            _: Not used
        """
        # Get settings
        settings = Settings.getInstance()
        
        # Get required settings
        api_key = settings.settings[Settings.API_KEY].value
        username = settings.settings[Settings.USERNAME].value
        password = settings.settings[Settings.PASSWORD].value
        
        # Get address settings - these are not directly defined in Settings class
        # Using defaults instead
        addr_locale = ""  # Default empty string
        addr_database = ""  # Default empty string
        
        # Get API settings
        srs = settings.settings[Settings.STREETSMART_SRS].value
        
        # Fix locale format - the API expects 'en-US' format instead of 'en_US'
        locale = settings.settings[Settings.LANGUAGE].value
        if locale and '_' in locale:
            locale = locale.replace('_', '-')
            
        # Default to en-US if no locale is set
        if not locale:
            locale = "en-US"  # Default locale
        
        # Get URLs
        config_url = settings.settings[Settings.CONFIGURATION_URL].value
        api_url = settings.settings[Settings.STREETSMART_LOCATION].value

        # Log the settings we're using (for debugging)
        logger.debug(f"API Settings: username={username}, api_key={api_key}")
        logger.debug(f"Locale: {locale}, SRS: {srs}")
        logger.debug(f"URLs: API={api_url}, Config={config_url}")
        
        # Prepare settings for API initialization
        if not username or not password or not api_key:
            userSettings = {
                "apiKey": ""
            }
        else:
            userSettings = {
                "username": username,
                "password": password,
                "apiKey": api_key
            }
            
        addressSettings = {
            "locale": addr_locale,
            "database": addr_database
        }
        
        configSettings = {
            "streetsmartApiUrl": api_url,
            "configUrl": config_url,
            "srs": srs,
            "locale": locale
        }

        message_load = json.dumps({
            "userSettings": userSettings,
            "addressSettings": addressSettings,
            "configSettings": configSettings
        })

        # Log the final JSON being sent
        logger.debug(f"Sending init message: {message_load}")

        # Initialize API with our settings
        self.streetsmart.sendToViewer("initapi|{}".format(message_load))
        
        # Add bidirectional communication support
        js_code = self._generate_communication_js()
        self.streetsmart.sendToViewer("execjs|{}".format(js_code))

    def _generate_communication_js(self):
        """
        Generate JavaScript code for bidirectional communication
        
        Returns:
            JavaScript code as string
        """
        # Add message handler to the window to process commands from QGIS
        js_code = """
        // Set up message handler for QGIS commands
        if (!window.qgisCommandHandler) {
            window.qgisCommandHandler = function(event) {
                // Check if we have the receiveQgisCommand function
                if (typeof window.receiveQgisCommand === 'function') {
                    try {
                        // Parse the command data
                        const commandData = event.data;
                        if (typeof commandData === 'string' && commandData.startsWith('qgis:')) {
                            // Extract the actual command part after the prefix
                            const command = commandData.substring(5);
                            // Process the command
                            window.receiveQgisCommand(command);
                        }
                    } catch (e) {
                        console.error('Error processing QGIS command:', e);
                    }
                }
            };
            
            // Add the event listener
            window.addEventListener('message', window.qgisCommandHandler);
            console.log('QGIS command handler initialized');
        }
        """
        return js_code
    

    def _generate_communication_js(self):
        """
        Generate JavaScript code for bidirectional communication
        
        Returns:
            JavaScript code as string
        """
        # More comprehensive initialization for panorama control
        js_code = """
        // Store the initial FOV value when a panorama is loaded
        if (!window.defaultHFov && window.panoramaViewer && window.panoramaViewer.getOrientation) {
            try {
                const orientation = window.panoramaViewer.getOrientation();
                if (orientation && orientation.hFov) {
                    window.defaultHFov = orientation.hFov;
                    console.log("Set default hFov to:", window.defaultHFov);
                }
            } catch (e) {
                console.error("Error getting initial FOV:", e);
            }
        }
        
        // Add rotation function
        window.rotatePanorama = function(yaw) {
            if (!window.panoramaViewer || !window.panoramaViewer.getOrientation) {
                console.error("Cannot rotate - panorama viewer not available");
                return;
            }
            
            try {
                console.log("Rotating panorama to:", yaw);
                const currentOrientation = window.panoramaViewer.getOrientation();
                const newOrientation = {...currentOrientation, yaw: parseFloat(yaw)};
                window.panoramaViewer.setOrientation(newOrientation);
                console.log("Rotation applied successfully");
            } catch (e) {
                console.error("Error rotating panorama:", e);
            }
        };
        
        // Add zoom function
        window.applyZoomToViewer = function(zoomFactor) {
            if (!window.panoramaViewer) {
                console.error("Cannot zoom - panorama viewer not available");
                return;
            }
            
            try {
                console.log("Setting zoom factor to:", zoomFactor);
                
                // Try different methods to apply zoom
                if (window.panoramaViewer.setZoom) {
                    // Direct zoom method
                    window.panoramaViewer.setZoom(parseFloat(zoomFactor));
                    console.log("Zoom applied via setZoom");
                    return;
                }
                
                if (window.panoramaViewer.getOrientation && window.panoramaViewer.setOrientation) {
                    const currentOrientation = window.panoramaViewer.getOrientation();
                    if (currentOrientation && currentOrientation.hFov) {
                        // Calculate new FOV based on zoom factor (inverse relationship)
                        const defaultHFov = window.defaultHFov || 70;
                        const newHFov = defaultHFov / parseFloat(zoomFactor);
                        
                        // Apply new orientation with adjusted FOV
                        window.panoramaViewer.setOrientation({
                            ...currentOrientation,
                            hFov: newHFov
                        });
                        console.log("Zoom applied via FOV adjustment");
                        return;
                    }
                }
                
                if (window.panoramaViewer.setCameraParameters) {
                    // Camera parameters method
                    window.panoramaViewer.setCameraParameters({ zoom: parseFloat(zoomFactor) });
                    console.log("Zoom applied via camera parameters");
                    return;
                }
                
                console.warn("No suitable method found to apply zoom");
            } catch (e) {
                console.error("Error applying zoom:", e);
            }
        };
        
        // Add message handler for commands from QGIS
        window.receiveQgisCommand = function(command) {
            console.log("Received command from QGIS:", command);
            
            const parts = command.split("|");
            if (parts.length < 2) {
                console.error("Invalid command format");
                return;
            }
            
            const commandType = parts[0];
            const commandData = parts[1];
            
            console.log("Command type:", commandType, "Data:", commandData);
            
            switch (commandType) {
                case "rotate":
                    window.rotatePanorama(commandData);
                    break;
                    
                case "zoom":
                    window.applyZoomToViewer(commandData);
                    break;
                    
                default:
                    console.error("Unknown command type:", commandType);
            }
        };
        
        // Set up message event listener
        window.addEventListener('message', function(event) {
            try {
                const data = event.data;
                if (typeof data === 'string' && data.startsWith('qgis:')) {
                    const command = data.substring(5); // Remove the 'qgis:' prefix
                    console.log("Processing QGIS command:", command);
                    window.receiveQgisCommand(command);
                }
            } catch (e) {
                console.error("Error processing message event:", e);
            }
        });
        
        console.log("Panorama control functions initialized");
        """
        return js_code